/**
 * Name: SimpleCVI_LJM.c
 * Auth: LabJack
 * Desc: Basic LabWindows/CVI example demonstrating LJM library usage.
 *       Tested in CVI version 5.5.
 *       Example code shows how to use the LJM_eReadNames and LJM_eWriteNames functions.
 *
 *       Code derived from the LJM C/C++ examples provided by LabJack. Download those for
 *       more C/C++ examples.
 *
 *       main, eReadNamesExample and eWriteNamesExample function code derived from
 *       from eReadNames.c amd eWriteNames.c .
 *       Other function code derived from LJM_Utilities.h .
 *
 *       To make a new project from the LJM C/C++ examples, the process is:
 *
 *       1) Make a new project.
 *
 *       2) Choose "Edit => Add Files To Project..." and add the desired C files.
 *
 *       3) Do the same and add the LJM lib file, LabJackM.lib. The lib file is normally located in
 *          "C:\Program Files (x86)\LabJack\Drivers" (64-bit Windows) or
 *          "C:\Program Files\LabJack\Drivers" (32-bit Windows) folder.
 *          If your application that you are building is 64-bits, use the lib in the
 *          "LabJack\Drivers\64bit" folder.
 *          This example builds an 32-bit application and uses the 32-bit lib. 
 *
 *       4) When including the LabJackM.h header file either change the include to use the full
 *          path to the file, or in your project add the path to the header file in the "Include Paths"
 *          settings (Options -> Include Paths). The path to the header file is the folder mentioned
 *          in step 3, "LabJack\Drivers".
 *          This example has the 32-bit and 64-bit Windows paths in the "Include Paths" settings. 
 *
 *       5) Do Ctrl+M to build the project.
 *		   
 *       6) Do Shift+F5 to run the project.
**/


/** Includes **/


#include <stdio.h>

#include <ansi_c.h> 

// Note for older LabWindows CVI versions and the LabJack.h header:
//   - If your compiler does not support "long long" variables, you will be unable to
//     to use the LJM_NumberToMAC amd LJM_MACToNumber functions. Comment the prototypes
//     out of the header file.
//   - For LJM functions with no parameters, you will need to add a void parameter.
//     For example, in the header file change:
//     LJM_ERROR_RETURN LJM_CloseAll();
//	   to:
//	   LJM_ERROR_RETURN LJM_CloseAll(void);
#include "LabJackM.h"


/** Function Prototypes **/


// Calls WaitForUser if this is compiled on Windows. This call is intended to be placed
// at the end of a program to prevent output from being lost in the case that it is being called
// from Visual Studio.
// Prompts the user to press enter
void WaitForUser(void);

// Print device info
void PrintDeviceInfoFromHandle(int handle);
void PrintDeviceInfo(int deviceType, int connectionType, int serialNumber, int ipAddressInt, int portOrPipe,
	int packetMaxBytes);
	
/**
 * Name: ErrorCheck
 * Desc: If err is not LJME_NOERROR, displays the error and exits the program.
 * Para: err, the error code being checked
 *       formattedDescription, a string representing what sort of operation returned err.
 *                             This can have formatting arguments in it, just like a string
 *                             passed to printf
**/	
void ErrorCheck(int err, const char * formattedDescription, ...);

/**
 * Name: ErrorCheckWithAddress
 * Desc: If err is not LJME_NOERROR, displays the error and exits the program. If errAddress
 *       is not negative, outputs the error address corresponding to err
 * Para: err, the error code being checked
 *       formattedDescription, a string representing what sort of operation returned err.
 *                             This can have formatting arguments in it, just like a string
 *                             passed to printf
 *       errAddress, the address corresponding to err. If this is negative, it will not be output.
 *                   Note that LJM functions such as LJM_eReadAddresses will not change the
 *                   ErrorAddress unless relevant
**/
void ErrorCheckWithAddress(int err, int errAddress, const char * description, ...);

/**
 * Desc: Prints the error if there is an errror
**/
void PrintErrorIfError(int err, const char * formattedDescription, ...);


/**
 * Desc: Demonstrates LJM_eReadNames usage.
**/
void eReadNamesExample(int handle);

/**
 * Desc: Demonstrates LJM_eWriteNames usage.
**/
void eWriteNamesExample(int handle);

#define INITIAL_ERR_ADDRESS -2  


/** main Function **/


int main()
{
	int err = 0;
	int handle = 0;

    // Open the first found device of any type
	err = LJM_Open(LJM_dtANY, LJM_ctANY, "LJM_idANY", &handle);
	ErrorCheck(err, "LJM_Open");

    // Display device information
	PrintDeviceInfoFromHandle(handle);

    printf("\n");
    
    // Run the eReadNames example.
    eReadNamesExample(handle);
    
    
    printf("\n");
    
    // Run the eWriteNames example.
    eWriteNamesExample(handle);
	
	// Close the device
	err = LJM_Close(handle);
	ErrorCheck(err, "LJM_Close");

	printf("\n");
    
    WaitForUser();

	return LJME_NOERROR;
}


/** Function Implementations **/


void eReadNamesExample(int handle)
{
    int err = 0;
    int i = 0;
	#define NUM_FRAMES_READ 3

	const char * NAMES[NUM_FRAMES_READ] = {"SERIAL_NUMBER", "PRODUCT_ID", "FIRMWARE_VERSION"};
	double values[NUM_FRAMES_READ] = {0.0, 0.0, 0.0};

	int errorAddress = INITIAL_ERR_ADDRESS;

	err = LJM_eReadNames(handle, NUM_FRAMES_READ, NAMES, values, &errorAddress);
	ErrorCheckWithAddress(err, errorAddress, "LJM_eReadNames");

	// Print results
	printf("Read:\n");
	for (i = 0; i<NUM_FRAMES_READ; i++) {
		printf("\t%s: %f\n", NAMES[i], values[i]);
	}
}

void eWriteNamesExample(int handle)
{
	int err = 0;
	int i = 0;
	int errorAddress = INITIAL_ERR_ADDRESS;

	#define NUM_FRAMES_WRITE 2

	const char * NAMES[NUM_FRAMES_WRITE] = {"DAC0", "TEST_UINT16"};
	double VALUES[NUM_FRAMES_WRITE] = {2.5, 12345};

	// Print what we're doing here
	printf("Writing:\n");
	for (i=0; i<NUM_FRAMES_WRITE; i++) {
		printf("\t%f to %s\n", VALUES[i], NAMES[i]);
	}

	err = LJM_eWriteNames(handle, NUM_FRAMES_WRITE, NAMES, VALUES, &errorAddress);
	ErrorCheckWithAddress(err, errorAddress, "LJM_eWriteNames");
}


void WaitForUser()
{
	printf("Press enter to continue\n");
	getchar();
}


const char * NumberToConnectionType(int connectionType)
{
	static const char * LJM_ctANY_string = "LJM_ctANY";
	static const char * LJM_ctUSB_string = "LJM_ctUSB";
	static const char * LJM_ctTCP_string = "LJM_ctTCP";
	static const char * LJM_ctETHERNET_string = "LJM_ctETHERNET";
	static const char * LJM_ctWIFI_string = "LJM_ctWIFI";
	static const char * unknown_string = "Unknown connection type";

	if (connectionType == LJM_ctANY)
		return LJM_ctANY_string;
	if (connectionType == LJM_ctUSB)
		return LJM_ctUSB_string;
	if (connectionType == LJM_ctTCP)
		return LJM_ctTCP_string;
	if (connectionType == LJM_ctETHERNET)
		return LJM_ctETHERNET_string;
	if (connectionType == LJM_ctWIFI)
		return LJM_ctWIFI_string;

	return unknown_string;
}


const char * NumberToDeviceType(int deviceType)
{
	static const char * LJM_dtANY_string = "LJM_dtANY";
	static const char * LJM_dtUE9_string = "LJM_dtUE9";
	static const char * LJM_dtU3_string = "LJM_dtU3";
	static const char * LJM_dtU6_string = "LJM_dtU6";
	static const char * LJM_dtT7_string = "LJM_dtT7";
	static const char * LJM_dtSKYMOTE_BRIDGE_string = "LJM_dtSKYMOTE_BRIDGE";
	static const char * LJM_dtDIGIT_string = "LJM_dtDIGIT";
	static const char * unknown_string = "Unknown device type";

	// These devices are not currently supported for LJM, but may be added
	static const int LJM_dtUE9 = 9;
	static const int LJM_dtU3 = 3;
	static const int LJM_dtU6 = 6;
	static const int LJM_dtSKYMOTE_BRIDGE = 1000;

	if (deviceType == LJM_dtANY)
		return LJM_dtANY_string;
	if (deviceType == LJM_dtUE9)
		return LJM_dtUE9_string;
	if (deviceType == LJM_dtU3)
		return LJM_dtU3_string;
	if (deviceType == LJM_dtU6)
		return LJM_dtU6_string;
	if (deviceType == LJM_dtT7)
		return LJM_dtT7_string;
	if (deviceType == LJM_dtSKYMOTE_BRIDGE)
		return LJM_dtSKYMOTE_BRIDGE_string;
	if (deviceType == LJM_dtDIGIT)
		return LJM_dtDIGIT_string;

	return unknown_string;
}


int IsTCP(int connectionType)
{
	if (connectionType == LJM_ctTCP ||
		connectionType == LJM_ctETHERNET ||
		connectionType == LJM_ctWIFI)
	{
		return 1;
	}
	return 0;
}


void PrintDeviceInfoFromHandle(int handle)
{
	int DeviceType, ConnectionType, SerialNumber, IPAddress, Port, MaxBytesPerMB;

	int err = LJM_GetHandleInfo(handle, &DeviceType, &ConnectionType, &SerialNumber,
		&IPAddress, &Port, &MaxBytesPerMB);

	ErrorCheckWithAddress(err, INITIAL_ERR_ADDRESS, "PrintDeviceInfoFromHandle (LJM_GetHandleInfo)");

	PrintDeviceInfo(DeviceType, ConnectionType, SerialNumber, IPAddress, Port, MaxBytesPerMB);
}


void PrintDeviceInfo(int deviceType, int connectionType, int serialNumber, int ipAddressInt, int portOrPipe,
	int packetMaxBytes)
{
	char ipAddressString[LJM_IPv4_STRING_SIZE];

	// Print
	printf("deviceType: %s\n", NumberToDeviceType(deviceType));
	printf("connectionType: %s\n", NumberToConnectionType(connectionType));
	printf("serialNumber: %d\n", serialNumber);

    if (IsTCP(connectionType)) {
		LJM_NumberToIP(ipAddressInt, ipAddressString);
		printf("IP address: %s\n", ipAddressString);
	}

	if (connectionType == LJM_ctUSB) {
		printf("pipe: %d\n", portOrPipe);
	}
	else {
		printf("port: %d\n", portOrPipe);
	}

	printf("The maximum number of bytes you can send to or receive from this device in one packet is %d bytes.\n",
		packetMaxBytes);
}


void PrintErrorAddressHelper(int errAddress)
{
	if (!(errAddress < 0))
		printf("\terror address: %d\n", errAddress);
}


typedef enum {
	ACTION_PRINT_AND_EXIT,
	ACTION_PRINT
} ErrorAction;


// The "internal" print function for ErrorCheck and ErrorCheckWithAddress
void _ErrorCheckWithAddress(int err, int errAddress, ErrorAction action,
	const char * description, va_list args)
{
	char errName[LJM_MAX_NAME_SIZE];
	if (err >= LJME_WARNINGS_BEGIN && err <= LJME_WARNINGS_END) {
		LJM_ErrorToString(err, errName);
		vfprintf (stdout, description, args);
		printf(" warning: \"%s\" (Warning code: %d)\n", errName, err);
		PrintErrorAddressHelper(errAddress);
	}
	else if (err != LJME_NOERROR)
	{
		LJM_ErrorToString(err, errName);
		vfprintf (stdout, description, args);
		printf(" error: \"%s\" (ErrorCode: %d)\n", errName, err);
		PrintErrorAddressHelper(errAddress);

		if (action == ACTION_PRINT_AND_EXIT) {
			printf("Closing all devices and exiting now\n");
			WaitForUser();
			LJM_CloseAll();
			exit(err);
		}
	}
}


void ErrorCheck(int err, const char * formattedDescription, ...)
{
	va_list args;

	va_start (args, formattedDescription);
	_ErrorCheckWithAddress(err, INITIAL_ERR_ADDRESS, ACTION_PRINT_AND_EXIT, formattedDescription, args);
	va_end (args);
}


void ErrorCheckWithAddress(int err, int errAddress, const char * description, ...)
{
	va_list args;

	va_start (args, description);
	_ErrorCheckWithAddress(err, errAddress, ACTION_PRINT_AND_EXIT, description, args);
	va_end (args);
}

